'use strict';
var lodash_1 = require('lodash');
var jsonapi_serializer_1 = require('jsonapi-serializer');
var utils_1 = require('./utils');
/**
 * Mapper class for Bookshelf sources
 */
var Bookshelf = (function () {
    /**
     * Standard constructor
     */
    function Bookshelf(baseUrl, serialOpts) {
        this.baseUrl = baseUrl;
        this.serialOpts = serialOpts;
    }
    /**
     * Maps bookshelf data to a JSON-API 1.0 compliant object
     *
     * The `any` type data source is set for typing compatibility, but must be removed if possible
     * TODO fix data any type
     */
    Bookshelf.prototype.map = function (data, type, bookOpts) {
        if (bookOpts === void 0) { bookOpts = {}; }
        var linkOpts = { baseUrl: this.baseUrl, type: type, pag: bookOpts.pagination };
        // Set default values for the options
        bookOpts = lodash_1.assign({ relations: true, enableLinks: true }, bookOpts);
        var info = { bookOpts: bookOpts, linkOpts: linkOpts };
        var template = utils_1.processData(info, data);
        var relationTypes = bookOpts.relationTypes || {};
        function typeForAttribute(attr) {
            // if typeForAttribute returns undefined, the serializer
            // omits the result and handles pluralization
            if (typeof relationTypes === 'object') {
                return relationTypes[attr];
            }
            else {
                return relationTypes(attr);
            }
        }
        // Override the template with the provided serializer options
        lodash_1.assign(template, { typeForAttribute: typeForAttribute }, this.serialOpts);
        // Return the data in JSON API format
        var json = utils_1.toJSON(data);
        return new jsonapi_serializer_1.Serializer(type, template).serialize(json);
    };
    return Bookshelf;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Bookshelf;
//# sourceMappingURL=index.js.map